/* ***********************************************************
 * This file was automatically generated on 2012-05-18.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.TimeUnit;


/**
 * Device for controlling Stacks and four Bricklets
 */
public class BrickMaster extends Device {
	private final static byte FUNCTION_GET_STACK_VOLTAGE = (byte)1;
	private final static byte FUNCTION_GET_STACK_CURRENT = (byte)2;
	private final static byte FUNCTION_SET_EXTENSION_TYPE = (byte)3;
	private final static byte FUNCTION_GET_EXTENSION_TYPE = (byte)4;
	private final static byte FUNCTION_IS_CHIBI_PRESENT = (byte)5;
	private final static byte FUNCTION_SET_CHIBI_ADDRESS = (byte)6;
	private final static byte FUNCTION_GET_CHIBI_ADDRESS = (byte)7;
	private final static byte FUNCTION_SET_CHIBI_MASTER_ADDRESS = (byte)8;
	private final static byte FUNCTION_GET_CHIBI_MASTER_ADDRESS = (byte)9;
	private final static byte FUNCTION_SET_CHIBI_SLAVE_ADDRESS = (byte)10;
	private final static byte FUNCTION_GET_CHIBI_SLAVE_ADDRESS = (byte)11;
	private final static byte FUNCTION_GET_CHIBI_SIGNAL_STRENGTH = (byte)12;
	private final static byte FUNCTION_GET_CHIBI_ERROR_LOG = (byte)13;
	private final static byte FUNCTION_SET_CHIBI_FREQUENCY = (byte)14;
	private final static byte FUNCTION_GET_CHIBI_FREQUENCY = (byte)15;
	private final static byte FUNCTION_SET_CHIBI_CHANNEL = (byte)16;
	private final static byte FUNCTION_GET_CHIBI_CHANNEL = (byte)17;

	public class ChibiErrorLog {
		public int underrun;
		public int crcError;
		public int noAck;
		public int overflow;

		public String toString() {
			return "[" + "underrun = " + underrun + ", " + "crcError = " + crcError + ", " + "noAck = " + noAck + ", " + "overflow = " + overflow + "]";
		}
	}

	/**
	 * Creates an object with the unique device ID \c uid. This object can
	 * then be added to the IP connection.
	 */
	public BrickMaster(String uid) {
		super(uid);

		expectedName = "Master Brick";

		bindingVersion[0] = 1;
		bindingVersion[1] = 1;
		bindingVersion[2] = 0;
	}

	/**
	 * Returns the stack voltage in mV. The stack voltage is the
	 * voltage that is supplied via the stack, i.e. it is given by a 
	 * Step-Down or Step-Up Power Supply.
	 */
	public int getStackVoltage() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_STACK_VOLTAGE);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_STACK_VOLTAGE, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getStackVoltage in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int voltage = IPConnection.unsignedShort(bb.getShort());

		semaphoreWrite.release();
		return voltage;
	}

	/**
	 * Returns the stack current in mA. The stack current is the
	 * current that is drawn via the stack, i.e. it is given by a
	 * Step-Down or Step-Up Power Supply.
	 */
	public int getStackCurrent() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_STACK_CURRENT);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_STACK_CURRENT, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getStackCurrent in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int current = IPConnection.unsignedShort(bb.getShort());

		semaphoreWrite.release();
		return current;
	}

	/**
	 * Writes the extension type to the EEPROM of a specified extension. 
	 * The extension is either 0 or 1 (0 is the on the bottom, 1 is the on on top, 
	 * if only one extension is present use 0).
	 * 
	 * Possible extension types:
	 * 
	 * \verbatim
	 *  :header: "Type", "Description"
	 *  :widths: 10, 100
	 *  "1", "Chibi"
	 *  "2", "RS485"
	 * \endverbatim
	 * 
	 * The extension type is already set when bought and it can be set with the 
	 * Brick Viewer, it is unlikely that you need this function.
	 * 
	 * The value will be saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 */
	public void setExtensionType(short extension, long exttype)  {
		ByteBuffer bb = ByteBuffer.allocate(9);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_SET_EXTENSION_TYPE);
		bb.putShort((short)9);
		bb.put((byte)extension);
		bb.putInt((int)exttype);

		ipcon.write(this, bb, FUNCTION_SET_EXTENSION_TYPE, false);
	}

	/**
	 * Returns the extension type for a given extension as set by 
	 * {@link com.tinkerforge.BrickMaster.setExtensionType}.
	 */
	public long getExtensionType(short extension) throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(5);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_EXTENSION_TYPE);
		bb.putShort((short)5);
		bb.put((byte)extension);

		ipcon.write(this, bb, FUNCTION_GET_EXTENSION_TYPE, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getExtensionType in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		long exttype = IPConnection.unsignedInt(bb.getInt());

		semaphoreWrite.release();
		return exttype;
	}

	/**
	 * Returns true if a Chibi Extension is available to be used by the Master.
	 */
	public boolean isChibiPresent() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_IS_CHIBI_PRESENT);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_IS_CHIBI_PRESENT, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for isChibiPresent in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		boolean present = (bb.get()) != 0;

		semaphoreWrite.release();
		return present;
	}

	/**
	 * Sets the address (1-255) belonging to the Chibi Extension.
	 * 
	 * It is possible to set the address with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 */
	public void setChibiAddress(short address)  {
		ByteBuffer bb = ByteBuffer.allocate(5);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_SET_CHIBI_ADDRESS);
		bb.putShort((short)5);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_ADDRESS, false);
	}

	/**
	 * Returns the address as set by {@link com.tinkerforge.BrickMaster.setChibiAddress}.
	 */
	public short getChibiAddress() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_CHIBI_ADDRESS);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_ADDRESS, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getChibiAddress in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Sets the address (1-255) of the Chibi Master. This address is used if the
	 * Chibi Extension is used as slave (i.e. it does not have a USB connection).
	 * 
	 * It is possible to set the address with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 */
	public void setChibiMasterAddress(short address)  {
		ByteBuffer bb = ByteBuffer.allocate(5);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_SET_CHIBI_MASTER_ADDRESS);
		bb.putShort((short)5);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_MASTER_ADDRESS, false);
	}

	/**
	 * Returns the address as set by {@link com.tinkerforge.BrickMaster.setChibiMasterAddress}.
	 */
	public short getChibiMasterAddress() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_CHIBI_MASTER_ADDRESS);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_MASTER_ADDRESS, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getChibiMasterAddress in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Sets up to 256 slave addresses. The address numeration has to be used 
	 * ascending from 0. For example: If you use the Chibi Extension in Master mode
	 * (i.e. the stack has an USB connection) and you want to talk to three other
	 * Chibi stacks with the IDs 17, 23, and 42, you should call with "(0, 17),
	 * (1, 23) and (2, 42)".
	 * 
	 * It is possible to set the addresses with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, they don't
	 * have to be set on every startup.
	 */
	public void setChibiSlaveAddress(short num, short address)  {
		ByteBuffer bb = ByteBuffer.allocate(6);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_SET_CHIBI_SLAVE_ADDRESS);
		bb.putShort((short)6);
		bb.put((byte)num);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_SLAVE_ADDRESS, false);
	}

	/**
	 * Returns the slave address for a given num as set by 
	 * {@link com.tinkerforge.BrickMaster.setChibiSlaveAddress}.
	 */
	public short getChibiSlaveAddress(short num) throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(5);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_CHIBI_SLAVE_ADDRESS);
		bb.putShort((short)5);
		bb.put((byte)num);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_SLAVE_ADDRESS, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getChibiSlaveAddress in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Returns the signal strength in dBm. The signal strength updates every time a
	 * packet is received.
	 */
	public short getChibiSignalStrength() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_CHIBI_SIGNAL_STRENGTH);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_SIGNAL_STRENGTH, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getChibiSignalStrength in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short signalStrength = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return signalStrength;
	}

	/**
	 * Returns underrun, CRC error, no ACK and overflow error counts of the Chibi
	 * communication. If these errors start rising, it is likely that either the
	 * distance between two Chibi stacks is becoming too big or there are
	 * interferences.
	 */
	public ChibiErrorLog getChibiErrorLog() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_CHIBI_ERROR_LOG);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_ERROR_LOG, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getChibiErrorLog in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		ChibiErrorLog obj = new ChibiErrorLog();
		obj.underrun = IPConnection.unsignedShort(bb.getShort());
		obj.crcError = IPConnection.unsignedShort(bb.getShort());
		obj.noAck = IPConnection.unsignedShort(bb.getShort());
		obj.overflow = IPConnection.unsignedShort(bb.getShort());

		semaphoreWrite.release();
		return obj;
	}

	/**
	 * Sets the Chibi frequency range for the Chibi Extension. Possible values are:
	 * 
	 * \verbatim
	 *  :header: "Type", "Description"
	 *  :widths: 10, 100
	 *  "0", "OQPSK 868Mhz (Europe)"
	 *  "1", "OQPSK 915Mhz (US)"
	 *  "2", "OQPSK 780Mhz (China)"
	 *  "3", "BPSK40 915Mhz"
	 * \endverbatim
	 * 
	 * It is possible to set the frequency with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 */
	public void setChibiFrequency(short frequency)  {
		ByteBuffer bb = ByteBuffer.allocate(5);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_SET_CHIBI_FREQUENCY);
		bb.putShort((short)5);
		bb.put((byte)frequency);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_FREQUENCY, false);
	}

	/**
	 * Returns the frequency value as set by {@link com.tinkerforge.BrickMaster.setChibiFrequency}.
	 */
	public short getChibiFrequency() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_CHIBI_FREQUENCY);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_FREQUENCY, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getChibiFrequency in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short frequency = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return frequency;
	}

	/**
	 * Sets the channel used by the Chibi Extension. Possible channels are
	 * different for different frequencies:
	 * 
	 * \verbatim
	 *  :header: "Frequency", "Possible Channels"
	 *  :widths: 40, 60
	 *  "OQPSK 868Mhz (Europe)", "0"
	 *  "OQPSK 915Mhz (US)", "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
	 *  "OQPSK 780Mhz (China)", "0, 1, 2, 3"
	 *  "BPSK40 915Mhz", "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
	 * \endverbatim
	 * 
	 * It is possible to set the frequency with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 */
	public void setChibiChannel(short channel)  {
		ByteBuffer bb = ByteBuffer.allocate(5);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_SET_CHIBI_CHANNEL);
		bb.putShort((short)5);
		bb.put((byte)channel);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_CHANNEL, false);
	}

	/**
	 * Returns the channel as set by {@link com.tinkerforge.BrickMaster.setChibiChannel}.
	 */
	public short getChibiChannel() throws IPConnection.TimeoutException {
		ByteBuffer bb = ByteBuffer.allocate(4);
		bb.order(ByteOrder.LITTLE_ENDIAN);
		bb.put((byte)stackID);
		bb.put((byte)FUNCTION_GET_CHIBI_CHANNEL);
		bb.putShort((short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_CHANNEL, true);

		byte[] answer = null;
		try {
			answer = answerQueue.poll(IPConnection.TIMEOUT_ANSWER, TimeUnit.MILLISECONDS);
			if(answer == null) {
				throw new IPConnection.TimeoutException("Did not receive answer for getChibiChannel in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(answer, 4, answer.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short channel = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return channel;
	}
}